@@ -13,15 +13,19 @@
 #
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+import os
 
-enable_logging = True
+if os.path.exists("/tmp/openvpn-applet.debug"):
+    enable_logging = True
+else:
+    enable_logging = False
 
 if enable_logging:
     fd = open('/tmp/openvpn-applet.log','w+')
 else:
     fd = None
 
-app_version = '0.5.6'
+app_version = '0.6.0'
 about_name = 'OpenVPN Applet'
 about_text = 'Applet for controlling OpenVPN connections'
 about_authors = 'Mikko Vartiainen'
@@ -36,7 +40,6 @@ import sys
 sys.path.insert(0, '/usr/lib/openvpn-applet')
 
 import glob
-import os
 import string
 from subprocess import Popen, PIPE, call
 import gtk
@@ -52,15 +55,13 @@ import time
 import conic
 
 def copy_file(file):
-    print "Copy: ", file
+    log("Copy: ", file)
     for f in file:
         if f != None:
-            print f
-            print file[f]
             call(['sudo','/usr/libexec/openvpn-applet-helper.py', 'copy', str(f), str(file[f])])
 
 def delete_files(file):
-    print "Delete: ", file
+    log("Delete: ", file)
     for f in file:
         if f != None:
             call(['sudo','/usr/libexec/openvpn-applet-helper.py', 'delete', f])
@@ -81,14 +82,13 @@ def openvpn_running(pid):
     pipe = p.stdout
     s = pipe.read().split()
     p.wait()
-    print s
     if s.count(str(pid)):
         return True
     else:
         return False
         
 class vpn():
-    def __init__(self, conf_file, conf_dir='/etc/openvpn'):
+    def __init__(self, conf_file, conf_dir='/etc/openvpn', auth_callback_fb=None):
         self.conf_file = conf_file
         self.name = string.split(os.path.basename(self.conf_file),'.')[0]
         self.pid_file = '/var/run/openvpn.'+self.name+'.pid'
@@ -97,29 +97,18 @@ class vpn():
         self.startcmd = '/usr/sbin/openvpn'
         self.socket_name = '/tmp/'+self.name+'.socket'
         self.socket = None
-        parameters = self.__read_conf()
-        log(parameters)
-        if parameters["auth-user-pass"] == True:
-            self.user_auth = True
-        else:
-            self.user_auth = False
-        
-        if parameters["askpass"] == True or parameters["auth-user-pass"] == True:
-            self.pass_auth = True      
-        else:
-            self.pass_auth = False
+        self.auth_callback_fb = auth_callback_fb
 
     def __socket_connect(self):
         if self.socket == None:
             self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
             try:
                 self.socket.connect(self.socket_name)
-            except:
-                log("socket.connect() failed")
+            except socket.error, (value,message):
+                log("socket.connect() failed:",value, message)
                 self.socket = None
+                raise socket.error(value,message);
                 return False
-            data, address = self.socket.recvfrom(256)
-            log("Connect: ", address, data)
             return True
         return False
 
@@ -128,7 +117,7 @@ class vpn():
             self.socket.close()
             self.socket = None
 
-    def __admin_command(self, command, get_response = True):
+    def __admin_command(self, command, get_response = True, disconnect = True, wait_for_ack = True):
         data = ''
         if self.socket == None: 
             if self.__socket_connect() == False:
@@ -136,20 +125,33 @@ class vpn():
         bytes_sent = self.socket.send(command)
         log("Command: "+command,bytes_sent)
         if (get_response):
-            while True:
-                chunk = self.socket.recv(256)
-                data = data+chunk
-                if data[:5] == "ERROR": break
-                if data[-5:] == "END\r\n": break
-                if data[:7] == "SUCCESS": break
-                if chunk == '': break
-            log("Response: "+data[-100:])
-            
-        self.__socket_disconnect()
+            data = self.__admin_recv(wait_for_ack)
+            log("Response:", data[-3:])
+
+        if disconnect:
+            self.__socket_disconnect()
         return data
 
+    def __admin_recv(self, wait_for_ack=False):
+        if self.socket == None: return
+
+        data = ''
+        while True:
+            chunk = self.socket.recv(4096)
+            #log(chunk)
+            data = data+chunk
+            lines = data.split("\r\n")
+            if wait_for_ack:
+                if data.find("SUCCESS") != -1: break
+                if data.find("ERROR") != -1: break
+                if data.find("END") != -1: break
+                if data.find(">FATAL") != -1: break
+            else:
+                if lines[-1] == '': break
+        return lines
+
     def get_log(self):
-        data = self.__admin_command("log all\n")
+        data = "\n".join(self.__admin_command("log all\n")[1:])
         while True:
             mindex = data.find("MANAGEMENT")
             if mindex == -1: break
@@ -159,15 +161,15 @@ class vpn():
             if start >= mindex or start == -1: start = 0
             data = data[:start+1]+data[end+1:]
     
-        return data[:-5]
+        return data[:-4]
 
     def get_statistics(self):
         data = self.__admin_command("status\n")
-        return data[:-5]
+        return "\n".join(data[1:-2])
 
     def get_state(self):
         data = self.__admin_command("state all\n")
-        return data[:-5]
+        return "\n".join(data[1:-2])
         
     def get_status(self):
         pid = self.get_pid()
@@ -179,7 +181,6 @@ class vpn():
             return False
         
     def get_pid(self):
-        print "pid_file: ",self.pid_file
         try:
             f = open(self.pid_file)
         except IOError:
@@ -189,21 +190,55 @@ class vpn():
         return int(pid)
         
     def start(self,user=None,passwd=None,enable_log=False):
-        command =["sudo",self.startcmd,"--daemon","--writepid",self.pid_file,"--config",self.conf_file,"--persist-key", "--management",self.socket_name,"unix"]
+        call(['sudo','/usr/libexec/add_default_route.sh'])
+        command =["sudo",self.startcmd,"--daemon","--writepid",self.pid_file,"--config",self.conf_file,"--persist-key", "--management",self.socket_name,"unix","--management-hold","--management-query-passwords", "--cd", self.conf_dir]
         if enable_log:
             command.extend(['--verb','6'])
         log(command)
         process = Popen(command, shell=False, stdin=PIPE, stdout=None, cwd=self.conf_dir)
         input = process.stdin
-              
-        if self.user_required and user != None:
-            print >>input,user
-
-        if self.pass_required and passwd != None:
-            print >>input,passwd
-
         input.close()
 
+        for i in range(1,10):
+            try:
+                self.__admin_command("hold off\n")
+            except socket.error:
+                time.sleep(0.2)
+            else:
+                break
+        response = self.__admin_command("hold release\n", disconnect=False)
+        auth_entered = []
+        for i in range(2):
+            response.extend(self.__admin_command("test 1\n", disconnect=False, wait_for_ack=False))
+            response.extend(self.__admin_command("test 1\n", disconnect=True, wait_for_ack=False))
+            for line in response:
+                if line[:9] == ">PASSWORD":
+                    if len(line.split()) > 3:
+                        (a, auth_name,auth_name2, auth_type) = line.split()
+                        auth_name = auth_name+" "+auth_name2
+                    else:
+                        (a, auth_name, auth_type) = line.split()
+                    auth_name = auth_name[1:-1]
+                    ask_username = False
+
+                    if auth_type == 'username/password':
+                        ask_username = True
+                        log("auth-user-pass", auth_name)
+                    if auth_type == 'password':
+                        ask_username = False
+                        log("askpass", auth_name)
+
+                    try:
+                        auth_entered.index(auth_name)
+                    except ValueError:
+                        (username,password,response_ok) = self.auth_callback_fb(ask_username, "OpenVPN Authentication: "+self.name+" "+auth_name)
+                        auth_entered.append(auth_name)
+                        if response_ok:
+                            if ask_username: self.__admin_command("username \""+auth_name+"\" \""+username+"\"\n")
+                            self.__admin_command("password \""+auth_name+"\" \""+password+"\"\n")
+                        else:
+                            self.stop()
+
         return process.returncode
         
     def stop(self):
@@ -220,41 +255,6 @@ class vpn():
     def get_name(self):
         return self.name
 
-    def user_required(self):
-        return self.user_auth
-        
-    def pass_required(self):
-        return self.pass_auth
-
-    def __read_conf(self):
-        keywords = ["askpass","auth-user-pass"]
-        parameters = dict.fromkeys(keywords,False)
-        
-        line = None
-        for line in fileinput.input(self.conf_file):
-            #print "line: ", line
-            #check for keywords
-            s = line.split()
-            if len(s) > 0:
-                for word in keywords:
-                    #print word
-                    if word == s[0]:
-                        print "MATCH: ", word,s
-                        #get filename
-                        if len(s) > 1:
-                            filename = line.split()[1]
-                            #check path
-                            if os.path.exists(filename):
-                                parameters[word] = filename
-                            elif os.path.exists(os.path.join(self.conf_dir, filename)):
-                                parameters[word] = filename
-                            else:
-                                parameters[word] = True
-                        else:
-                            parameters[word] = True
-
-        return parameters
-    
 class vpn_control():
     def read_vpn_conf(self, dir, ext):
         vpn_conf = []
@@ -278,10 +278,10 @@ class vpn_control():
         self.vpn_list = []
         for v in self.read_vpn_conf('/etc/openvpn',['conf','ovpn']):
             log(v)
-            self.vpn_list.append(vpn(v))
+            self.vpn_list.append(vpn(v, auth_callback_fb=user_pass_dlg))
         for v in self.read_vpn_conf('/home/user/.openvpn-applet',['conf','ovpn']):
             log(v)
-            self.vpn_list.append(vpn(v,'/home/user/.openvpn-applet'))
+            self.vpn_list.append(vpn(v,'/home/user/.openvpn-applet',auth_callback_fb=user_pass_dlg))
 
     def start(self, index,user=None,passwd=None,enable_log=False):
         self.vpn_list[index].start(user,passwd)
@@ -301,12 +301,6 @@ class vpn_control():
         else:
             return -1
 
-    def user_required(self,index):
-        return self.vpn_list[index].user_required()
-        
-    def pass_required(self,index):
-        return self.vpn_list[index].pass_required()
-        
     def get_vpn_count(self):
         return len(self.vpn_list)
 
@@ -418,24 +412,15 @@ class openvpn_applet(hildondesktop.StatusMenuItem):
 
         if vpn_number >= 0  and vpn_number < self.vc.get_vpn_count():
             if self.vc.status(vpn_number) == False:
-                if self.vc.user_required(vpn_number) and self.vc.pass_required(vpn_number):
-                    (user,passwd,cont) = user_pass_dlg(True)
-                elif self.vc.pass_required(vpn_number):
-                    (user,passwd,cont) = user_pass_dlg(False)
-                else:
-                    user   = None
-                    passwd = None
-                    cont = True
-                if cont:
-                    self.banner = hildon.hildon_banner_show_information(self, "", "Starting Openvpn Connection")
-                    self.vc.start(vpn_number,user,passwd)
+                self.banner = hildon.hildon_banner_show_information(self, "", "Starting Openvpn Connection")
+                self.vc.start(vpn_number,None,None)
             else:
                 self.banner = hildon.hildon_banner_show_information(self,"", "Stopping Openvpn Connection")
                 self.vc.stop(vpn_number)
             self.timeout = 10
             self.timer = gobject.timeout_add(500, self.update_button_timer)
         elif vpn_number == self.vc.get_vpn_count():
-            print "open settings"
+            log("open settings")
             self.open_settings(self)
 
     def connection_cb(self, connection, event):
@@ -448,6 +433,9 @@ class openvpn_applet(hildondesktop.StatusMenuItem):
         
         if status == conic.STATUS_CONNECTED:
             log("CONNECTED",iap_id, bearer, status, error)
+            if bearer == "GPRS" and self.vc != None and self.vc.get_active_connections_number() > 0:
+                log("add_default_route", self.vc.get_active_connections_number())
+                call(['sudo','/usr/libexec/add_default_route.sh'])
             if self.vc != None:
                 self.vc.restart_connected()
         elif status == conic.STATUS_DISCONNECTED:
@@ -455,16 +443,16 @@ class openvpn_applet(hildondesktop.StatusMenuItem):
         elif status == conic.STATUS_DISCONNECTING:
             log("(DISCONNECTING",iap_id, bearer, status, error)
 
-def user_pass_dlg(ask_passwd):
+def user_pass_dlg(ask_passwd, message):
     if ask_passwd:
         dlg = hildon.LoginDialog(gtk.Window())
-        dlg.set_message("OpenVPN Authentication")
+        dlg.set_message(message)
         response = dlg.run()
         user_pass_resp = (dlg.get_username(),dlg.get_password())
         dlg.destroy()
     else:
         dlg = hildon.GetPasswordDialog(gtk.Window(),False)
-        dlg.set_message("OpenVPN Authentication")
+        dlg.set_message(message)
         response = dlg.run()
         user_pass_resp = (None,dlg.get_password())
         dlg.destroy()
@@ -580,7 +568,7 @@ class OpenvpnAppletSettings(gtk.Dialog):
                 else:
                     files_to_copy[f] =  all_files[f]
             if len(files_to_overwrite) > 0:
-                print "Files to overwrite:", files_to_overwrite
+                log("Files to overwrite:", files_to_overwrite)
                 mstr = ""
                 for f in files_to_overwrite_message:
                     mstr = mstr+"\n"+f
@@ -626,18 +614,9 @@ class OpenvpnAppletSettings(gtk.Dialog):
         if vpn_number >= 0:
             log("Item selected", vpn_number)
             vpn = self.vpn_manager.vpn_list[vpn_number]
-            if vpn.user_required() and vpn.pass_required():
-                (user,passwd,cont) = user_pass_dlg(True)
-            elif vpn.pass_required():
-                (user,passwd,cont) = user_pass_dlg(False)
-            else:
-                user   = None
-                passwd = None
-                cont = True
-            if cont:
-                if vpn.get_status():
-                    vpn.stop()
-                vpn.start(user,passwd,True)
+            if vpn.get_status():
+                vpn.stop()
+            vpn.start(None,None,True)
 
             dlg = OpenvpnAppletTestConnection(parent=self, vpn_conn=vpn)            
             dlg.run()        
